#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>

#define L (1024)
#define M (1024)
#define N (1024)

void matmul(float *A, float *B, float *C,
	    int l, int m, int n)
{
    int i, j, k;
    for (i = 0; i < l; i++) {
        for (j = 0; j < n; j++) {
            float sum = 0.0;
            for (k = 0; k < m; k++) {
                sum += A[i * m + k] * B[k * n + j];
            }
            C[i*n+j] = sum;
        }
    }
}

void alloc_matrix(float **m_h, int h, int w)
{
    *m_h = (float *)malloc(sizeof(float) * h * w);
}

void init_matrix(float *m, int h, int w)
{
    int i, j;
    for (i = 0; i < h; i++)
        for (j = 0; j < w; j++)
            m[i * w + j] = (float)random();
}

double get_elapsed_time(struct timeval *begin, struct timeval *end)
{
    return (end->tv_sec - begin->tv_sec) * 1000
            + (end->tv_usec - begin->tv_usec) / 1000.0;
}

int main(int argc, char *argv[])
{
    float *Ah, *Bh, *Ch;
    struct timeval t1, t2;

    // prepare matrix A
    alloc_matrix(&Ah, L, M);
    init_matrix(Ah, L, M);

    // do it again for matrix B
    alloc_matrix(&Bh, M, N);
    init_matrix(Bh, M, N);

    // allocate spaces for matrix C
    alloc_matrix(&Ch, L, N);

    gettimeofday(&t1, NULL);

    // call matmul function
    matmul(Ah, Bh, Ch, L, M, N);

    gettimeofday(&t2, NULL);
    printf("Elapsed time: %f msec\n", get_elapsed_time(&t1, &t2));

    return 0;
}



